#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use File::Basename;

################################################################################
sub usage {
  my $myname = `basename $0`;
  print STDERR << "EOF";
Usage: $myname [-togmpdsc] trascript1 trascript2

example: $myname app_version1.T app_version2.T

-t Ignore type
-o Ignore owner
-g Ignore group
-m Ignore mode (permissions)
-p Ignore mode (permissions)
-d Ignore date
-s Ignore size
-c Ignore checksum
-i Print lines that are identical

EOF
    exit 1;
}

####################################
# Get ARGs
#
use Getopt::Std;

getopts("a",\%options);

my $default_missing_perms = "0755 0 0";
my $first_stack = "";
my $absolute_paths = 0;

my $check_bad_paths = 1;
my $check_uids = 1;
my $check_gids = 1;
my $check_world_writable = 1;
my $check_missing_parents = 1;

$absolute_paths = 1 if defined $options{"a"};

$t1 = $ARGV[0];

####################################
# Read the transcript
#
my @bad_paths = ();
my @uids = ();
my @gids = ();
my @gids_writable = ();
my @world_writable = ();
my @missing_parent_dirs = ();

if ( open T1, "< $t1" ) {
  
  my $directory_stack = $first_stack;
  my $last_missing_path = "";

  while ( <T1> ) {
    
    my $line = $_;

    %radmind_line = parse_radmind_line($_);

    if ( defined $radmind_line{comment} ) { next; }

    ####################################
    # Check for bad paths
    #
    if ( $check_bad_paths ) {
      if ( $absolute_paths ) {
        if ( ! $radmind_line{'absolutepath'} ) {
          # save paths that are listed wrong
          push @bad_paths, $line; # need ."\n";???
        }
      } else {
        if ( ! $radmind_line{'relativepath'} ) {
          # save paths that are listed wrong
          push @bad_paths, $line; # need ."\n";???
        }
      }
    }

    ####################################
    # Check permissions
    #
    if ( $radmind_line{filetype} ne "l" ) {

      ####################################
      # Find writable & uid & gid
      #

      if ( $check_uids ) {

        # save the uid
        if ( ! grep /^$radmind_line{'owner'}$/, @uids ) {
          push @uids, $radmind_line{'owner'};
        }
      }

      if ( $check_gids ) {

        # save the gid
        if ( ! grep /^$radmind_line{'group'}$/, @gids ) {
          push ( @gids, $radmind_line{'group'} );
        }

        # find group writable
        $group_perm = substr $radmind_line{'perms'}, 2, 1;
        if ( $group_perm == 2 or $group_perm == 3 or $group_perm == 6 or $group_perm == 7 ) {
          if ( ! grep /^$radmind_line{'group'}$/, @gids_writable ) {
            push @gids_writable, $radmind_line{'group'};
          }
        }

      }

      if ( $check_world_writable ) {

        # then look for world writable
        $world_perm = substr $radmind_line{'perms'}, 3, 1;
        if ( $world_perm == 2 or $world_perm == 3 or $world_perm == 6 or $world_perm == 7 ) {

          my $flag = 1;
          foreach $j ( @managedSpace ) {
            if ( ( index $radmind_line{'path'}, $j ) == 3 ) {
              $flag = 0;
              last;
            }
          }
          if ($flag) {
            push @world_writable, $line; # ."\n";
          }

        }

      }
    }

    ####################################
    # Check for missing parent dirs
    #
    if ( $check_missing_parents and $radmind_line{"path"} ne "" and $radmind_line{"path"} ne "/" and $radmind_line{"path"} ne "." ) {
    
      my $parent_dir = dirname ( $radmind_line{"path"} );

      # Check to see if the path is missing
      if ( index ($directory_stack, $parent_dir) < 0 ) {
        
        # Check the path to see if more is missing from it
        my $bla = dirname ( $parent_dir );
        @also_missing = ();
        while ( index ($last_missing_path, $bla) < 0 and $bla ne "." and $bla ne "/" ) {
          push  @also_missing, $bla;
          $bla = dirname ( $bla );
        }
        
        # Print any of the parent dirs that are also missing
        while ( $i = pop @also_missing ) {
          push @missing_parent_dirs, $i;
        }

        # Check the parent of the current path
        push @missing_parent_dirs, $parent_dir;

        $last_missing_path = $parent_dir;
        $directory_stack = $parent_dir;
      }

      if ( $radmind_line{"filetype"} eq "d" ) {
        $directory_stack = $radmind_line{"path"};
      }
    }

  }
  close T1;

  ####################################
  # Notify of problems
  #
  if ( $#uids >= 0 ) {
    print "The following UID's are used: @uids (/Applications, /System, etc should not be 501 but 0)\n";
  }
  if ( $#gids >= 0 ) {
    print "The following GID's are used: @gids (/Applications, /System, etc should not be 501 but 0 when possible)\n";
  }
  if ( $#gids_writable >= 0 ) {
    print "The following GID's have write permissions: @gids_writable (20 is staff, 80 is admin)\n";
  }

  if ( $#world_writable >= 0 ) {
    print "\nWorld Writable:\n\n";
    foreach $i ( @world_writable ) {
      print "$i";
    }
  }


  if ( $#missing_parent_dirs >= 0 ) {
    print "\nMissing parent dirs:\n\n";
    foreach $i ( @missing_parent_dirs ) {
      print "d $i $default_missing_perms\n";
    }
  }

  if ( $#bad_paths >= 0 ) {
    $asdf = ($absolute_paths)?"'/'":"'./'";
    print "\nThe following paths should begin with $asdf\n\n";
    foreach $i ( @bad_paths ) {
      print "$i";
    }
  }


} else {
  die "Could not open $t1: $!";
}

################################################################################
sub parse_radmind_line {
  my ( $line ) = @_;

  my %radmind_line = ();

  # check for comment
  if ( ( substr $line, 0, 1 ) eq "#" ) {
    $radmind_line{'comment'} = substr $line, 1, length ($line)-1;
    return %radmind_line;
  }
  
  my @theLine = split (' ', $line);

  # check for minus
  if ( $theLine[0] eq "-" ) {
    shift @theLine;
    $radmind_line{'minus'} = 1;
  }

  # check for plus (even needed?
  if ( $theLine[0] eq "+" ) {
    shift @theLine;
    $radmind_line{'plus'} = 1;
  }

  $radmind_line{'filetype'} = shift @theLine;

  $radmind_line{'path'} = shift @theLine;
  $radmind_line{'realpath'} = unescape_radmind_path ( $radmind_line{'path'} );
  if ( ( substr $radmind_line{'path'}, 0, 1 ) eq "." ) {
    $radmind_line{'relativepath'} = 1;
  } elsif ( ( substr $radmind_line{'path'}, 0, 1 ) eq "/" ) {
    $radmind_line{'absolutepath'} = 1;
  }

  if ($radmind_line{'filetype'} ne "l"  ) {

    $radmind_line{'perms'} = shift @theLine;
    $radmind_line{'owner'} = shift @theLine;
    $radmind_line{'group'} = shift @theLine;

    if ($radmind_line{'filetype'} ne "d"  ) {

      $radmind_line{'modtime'} = shift @theLine;
      $radmind_line{'size'} = shift @theLine;
      $radmind_line{'checksum'} = shift @theLine;

    } elsif ( $#theLine > 0 ) {
    
      $radmind_line{'appleinfo'} = shift @theLine;
    
    }

  } else {

    $radmind_line{'linkpath'} = shift @theLine;

  }

  return %radmind_line;

}

sub unescape_radmind_path {
  my ( $path ) = @_;
  $path =~ s/\\b/ /g;
#  my $path =~ s/\\t/\t/g;; # convert tabs
#  my $path =~ s/\\n/\n/g;; # convert newline
#  my $path =~ s/\\r/\r/g;; # convert return
#  my $path =~ s/\\/\/g;; # convert backslashes
  return $path;
}
