#!/usr/bin/perl

################################################################################
# rsync_command_file.pl
#
# This script rsyncs a master radmind server to a slave
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
#
################################################################################

if ( ! defined $ARGV[1] ) {
  print "Usage: $0 dest_server command_file [command_file...]";
  exit 1;
}

################################################################################
# Change these
#
$source_base = "/var/radmind";
$destination_base = "$ARGV[0]:/var/radmind";
$logger = "/usr/bin/logger -i -t $0"; # Your logger
$noisy_log = 0;
$rsync_output = "/tmp"; # this should be a folder
$rsync_output_backup_number = 5;

################################################################################
# Internal stuff
#

$options = "--rsh=ssh --hard-links --progress --delete";                 
$rsync_command = "/usr/bin/rsync -vrl $options ";   

$last_source = "bla"; # don't change this
$output_file = "$rsync_output/rsync.out";
$error_file = "$rsync_output/rsync.err";
$error_bool = 0;
$last_error_message = "";

################################################################################
# Log
#

system "$logger \"Started.\""; 

if ( ! -d $rsync_output ) {
  system "$logger \"NO valid log location, exiting.\""; 
  exit 1;
}

# Handle the log file
roll_log ( "$output_file", "$rsync_output_backup_number" );
roll_log ( "$error_file", "$rsync_output_backup_number" );

################################################################################
# Get command files to sync
#

@rsync_these_folders = (); # This is the list.  Everything is relative to /var/radmind

@parse_me = @ARGV;
@k_files = ();
shift @parse_me;

if ( ! -e $source_base ) {
  print STDERR "$source_base does not exist.\n";
  exit 1;
}
foreach $i (@parse_me) {
  if ( ! -e $i ) {
    print STDERR "$i does not exist.\n";
  } else {
    push (@k_files, $i);
    #print "parsing $i\n";
    @to_parse = parse_command_file ($i,"$source_base/command");
    foreach $j ( @to_parse ) {
      for (@parse_me) { $parse_me_copy{$_} = 1; }
      if ( ! $parse_me_copy{$j} ) {
        #print "adding $j\n";
        push (@parse_me, $j);
        #print "adding $j\n";
        $j = reg_ed($j);
        push (@rsync_these_folders,$j);
      } else {
        #print "added $j already\n";
      }
    }
  }
}
if ( $#k_files < 0 ) {
  print STDERR "No command files found.\n";
  exit 1;
}

sub parse_command_file {
  my ( $k_file, $path ) = @_;
  $contents = `cat $k_file | grep "^k"`;
  my @lines = split ('\n', $contents);
  my @files = ();
  foreach my $i ( @lines ) {
    @k_name = split (' ', $i);
    push (@files, $path."/$k_name[1]");
  }
  return @files;
}

################################################################################
# Get overloads to sync
#

@o_files = ();

foreach $k_file (@k_files) {
  $contents = `cat $k_file | grep -E "(^p|^n)"`;
  my @lines = split ('\n', $contents);
  my @files = ();
  foreach my $j ( @lines ) {
    @o_name = split (' ', $j);
    for (@o_files) { $o_files_copy{$_} = 1; }
    if ( ! $o_files_copy{"$o_name[1]"} ) {
      #print "$o_name[1]\n";
      push (@o_files, "$o_name[1]");
      push (@rsync_these_folders, "transcript/$o_name[1]");
      push (@rsync_these_folders, "file/$o_name[1]");
    }
  }
}

################################################################################
# Get overloads to sync
#

open ( LOG, ">>$output_file" );
print LOG `date`;
print LOG "Rsnyc starting.  Here is what I am going to sync:\n";
foreach $i ( @rsync_these_folders ) {
  print LOG "$source_base/$i\n";
}
close ( LOG );

################################################################################
# Run rsync
#
foreach $i ( @rsync_these_folders ) {
  run_rsync ( $rsync_command, "$source_base/$i", "$destination_base/$i" );
}

################################################################################
# Clean up
#

system "$logger \"Finished.\"";

################################################################################
sub run_rsync {
  my ( $rsync_command, $source, $destination ) = @_;

  # Do GUI message
  if ( -e $cl_to_gui_messenger ) {
    if ( $last_source ne "bla" ) {
      killCLGUI ( "$cl_to_gui_messenger", "root", "$last_source" );
    }
    $last_source = $source;
    system "\"$cl_to_gui_messenger\" \"Running Rsync, please do not change any radmind stuff!\" \"$source\" &";
  }

  # check the to see if it is a directory and append "/" if it is...
  if ( -d $source ) {
    $source .= "/";
    $destination .= "/";
  }

  # Print log info
  open ( LOG, ">>$output_file" );
  print LOG `date`;
  print LOG "$rsync_command $source $destination\n\n";

  # Run Rsync
  if ( $noisy_log ) {
    open ( COMMAND, "$rsync_command $source $destination 2>> $error_file |" );
    while ( <COMMAND> ) {
      print LOG;
    }
    close ( COMMAND );
  } else {
    system "$rsync_command $source $destination 2>> $error_file";  
  }

  # Check for errors
  $result = $? >> 8;
  if ( $result > 0 ) {
    $error_bool = 1;
    system "$logger \"Error: $source\"";
    $last_error_message = $source;
    print LOG "There was an error.";
  }
  close ( LOG );

}

################################################################################
# This script renames $path_to_log so that the lowest number is always the
# newest.  The oldest that is greater than $number_of_backups is deleted.
#  
sub roll_log {
  my ( $path_to_log, $number_of_backups ) = @_;
  if ( -e $path_to_log ) {
    if ( -e $path_to_log.".bak$number_of_backups" ) {    
      unlink $path_to_log.".bak$number_of_backups";
    }
    for ( $i = $number_of_backups ; $i > 1 ; $i-- ) {
      if ( -f $path_to_log.".bak".($i-1)) {
        system "/bin/mv -f \"$path_to_log".".bak".($i-1)."\" \"$path_to_log".".bak$i\"";
      }
    }
    system "/bin/mv -f \"$path_to_log\" \"$path_to_log".".bak1\"\n";
  } 
}

sub reg_ed {
  my ($j) = @_;
  $j =~ s\/var/radmind/\\;
  return $j
}

